__title__ = 'django-fobi'
__version__ = '0.9.6'
__build__ = 0x000068
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2016 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'

default_app_config = 'fobi.apps.Config'
