{% load i18n fobi_tags %}
                <div class="alert-box secondary alert-box-larger">
                    {% blocktrans %}Fields marked with <span class="required-field">*</span> are required{% endblocktrans %}
                </div>

            {% block form_non_field_and_hidden_errors %}
                {% get_form_hidden_fields_errors form as form_hidden_fields_errors %}
                {% if form.non_field_errors or form_hidden_fields_errors %}
                    {% include fobi_theme.form_non_field_and_hidden_errors_snippet_template %}
                {% endif %}
            {% endblock form_non_field_and_hidden_errors %}

            {% comment %}
            {% if form.non_field_errors %}
                <div class="row">
                    {{ form.non_field_errors.as_ul }}
                </div>
            {% endif %}
            {% endcomment %}

            {% for field in form %}

                {% if field.is_hidden %}
                    <div class="hidden">{{ field }}</div>
                {% else %}
                    <div class="row{% if field.errors %} error{% endif %}{% if field.is_hidden %} hidden{% endif %}">
                        <label class="large-2 columns" for="{{ field.id_for_label }}">
                            {{ field.label }}{% if field.field.required %}<span class="required-field">*</span>{% endif %}
                        </label>

                        <div class="large-10 columns">
                            {% get_form_field_type field as form_field_type %}

                            {% if form_field_type.is_radio %}
                                {% for choice_value, choice_label in field.field.choices %}
                            <div class="radio">
                              <label for="{{ field.html_name }}_{{ forloop.counter0 }}">
                                <input type="radio" id="{{ field.html_name }}_{{ forloop.counter0 }}" name="{{ field.html_name }}" value="{{ choice_value }}"{% if field.value == choice_value %}checked{% endif %}>
                                {{ choice_label }}
                              </label>
                            </div>
                                {% endfor %}
                            {% elif form_field_type.is_checkbox_multiple %}
                                {% for choice_value, choice_label in field.field.choices %}
                            <div class="checkbox">
                              <label for="{{ field.html_name }}_{{ forloop.counter0 }}">
                                <input type="checkbox" id="{{ field.html_name }}_{{ forloop.counter0 }}" name="{{ field.html_name }}" value="{{ choice_value }}"{% if choice_value in field.value %}checked{% endif %}>
                                {{ choice_label }}
                              </label>
                            </div>
                                {% endfor %}
                            {% else %}
                                {{ field }}
                            {% endif %}

                            {% if field.errors %}
                            <span class="error">
                                {% for field_error in field.errors %}
                                    {{ field_error }}
                                {% endfor %}
                            </span>
                            {% endif %}

                            {% if field.help_text %}
                            <span class="help-block">{{ field.help_text|safe }}</span>
                            {% endif %}
                        </div>
                    </div>
                {% endif %}

            {% endfor %}
